/*
* This file is part of Converseen, an open-source batch image converter
* and resizer.
*
* (C) Francesco Mondello 2009-2012
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* Contact e-mail: Faster <faster3ck@gmail.com>
*
*/

#include <string>
#include <iostream>
#include "mainwindowimpl.h"
#include "dialogoptions.h"
#include "dialoginfo.h"
#define INIFILENAME ".converseen.conf"

using namespace Magick;
using namespace std;

// TODO: Inserire funzione per sostituire tutte le immagini

MainWindowImpl::MainWindowImpl(QWidget * parent, Qt::WFlags f)
        : QMainWindow(parent, f)
{
    setupUi(this);

    setAcceptDrops(true);

    convertThread = new Converter(this);
    dlgCStatus = new DialogConversionStatus();

    connect(pushOpen, SIGNAL(clicked()), this, SLOT(openFiles()));
    connect(buttonAdd, SIGNAL(clicked()), this, SLOT(addFiles()));
    connect(buttonConvert, SIGNAL(clicked()), this, SLOT(elabora()));
    connect(bntSelDir, SIGNAL(clicked()), this, SLOT(openOutDirectory()));
    connect(checkSameDir, SIGNAL(stateChanged(int)), this, SLOT(setCurrentDirectory()));
    connect(pushRemove, SIGNAL(clicked()), this, SLOT(removeItems()));
    connect(pushRemoveAll, SIGNAL(clicked()), this, SLOT(removeAllItems()));
    connect(btnFormatOptions, SIGNAL(clicked()), this, SLOT(setQuality()));
    connect(treeWidget, SIGNAL(itemSelectionChanged()), this, SLOT(onItemSelection()));

    connect(spin_geoWidth, SIGNAL(valueChanged(double)), this, SLOT(relativeSizeW(double)));
    connect(spin_geoHeight, SIGNAL(valueChanged(double)), this, SLOT(relativeSizeH(double)));

    connect(comboResizeValues, SIGNAL(currentIndexChanged(QString)), this, SLOT(selectGeometryUnit(QString)));

    connect(checkRename, SIGNAL(stateChanged(int)), this, SLOT(enableRenameLine()));
    connect(checkShowPreview, SIGNAL(stateChanged(int)), this, SLOT(enableShowPreview()));

    connect(pushReset, SIGNAL(clicked()), this, SLOT(onPushResetClick()));

    connect(convertThread, SIGNAL(nextConversion(int)), this, SLOT(nextConversion(int)));
    connect(convertThread, SIGNAL(requestOverwrite(QString)), this, SLOT(overwriteDialog(QString)));
    connect(convertThread, SIGNAL(errorMessage(QString)), this, SLOT(errorMessage(QString)));

    connect(dlgCStatus, SIGNAL(stopProcess()), this, SLOT(stopProcess()));

    connect(checkOverwrite, SIGNAL(stateChanged(int)), this, SLOT(saveOverwritingSettings()));

    createActions();
    setupMenu();
    createContextMenu();

#ifdef Q_WS_WIN
    QString iniPath;
    if ((QSysInfo::windowsVersion() == QSysInfo::WV_VISTA) || (QSysInfo::windowsVersion() == QSysInfo::WV_WINDOWS7)) {
        iniPath = QDir::homePath() + "/AppData/Roaming/Converseen";
		QDir winDir;
        winDir.mkpath(iniPath);
    }
#endif

    loadFormats();

    comboResizeValues->addItems((QStringList() << "%" << "px"));
    resetDisplays();

    adjustSize();
    dockWidget->adjustSize();

    treeWidget->header()->setResizeMode(0, QHeaderView::ResizeToContents);

    loadOptions();
}

void MainWindowImpl::resizeEvent(QResizeEvent *event)
{
    settings->setValue("Options/WindowSize", saveGeometry());
    event->accept();
}

void MainWindowImpl::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasFormat("text/uri-list"))
        event->acceptProposedAction();
}

void MainWindowImpl::dropEvent(QDropEvent *event)
{
    QList<QUrl> urls = event->mimeData()->urls();
    if (urls.isEmpty())
        return;

    QStringList fileNames;
    QFileInfo fi;

    for (int i = 0; i < urls.count(); i++) {
        fi = urls.at(i).toLocalFile();

		if (readableFomats.contains(fi.suffix(), Qt::CaseInsensitive))
			fileNames << urls.at(i).toLocalFile().toLocal8Bit();
    }

    if (!fileNames.isEmpty())
        loadFiles(fileNames);
}

void MainWindowImpl::importListFromArgv(QString fileName)
{
    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QString text = file.readAll();
    QStringList rawlist = text.split("\n");

    QStringList list;

    for (int i = 0; i < rawlist.count(); i++) {
        QFileInfo fi(rawlist.at(i));
        if (readableFomats.contains(fi.suffix(), Qt::CaseInsensitive))
            list << rawlist.at(i);
    }

    loadFiles(list);
}

void MainWindowImpl::createActions()
{
    /* "File" actions*/
    actionOpenFiles = new QAction(tr("Open images to convert"), this);
    actionOpenFiles->setShortcut(tr("Ctrl+O"));
    connect(actionOpenFiles, SIGNAL(triggered()), this, SLOT(openFiles()));

    actionAddFiles = new QAction(tr("Add other images to convert"), this);
    actionAddFiles->setShortcut(tr("Ctrl+Alt+O"));
    connect(actionAddFiles, SIGNAL(triggered()), this, SLOT(addFiles()));

    actionShowOptions = new QAction(tr("Settings"), this);
    connect(actionShowOptions, SIGNAL(triggered()), this, SLOT(editSettings()));

    actionClose = new QAction(tr("&Exit"), this);
    actionClose->setShortcut(tr("Ctrl+Q"));
    connect(actionClose, SIGNAL(triggered()), this, SLOT(close()));

    /* "Edit" actions */
    actionCheckItems = new QAction(tr("&Check Selected Items"), this);
    //actionCheckItems->setShortcut(tr("Ctrl+A"));
    connect(actionCheckItems, SIGNAL(triggered()), this, SLOT(checkItems()));

    actionCheckAllItems = new QAction(tr("&Check All Items"), this);
    actionCheckAllItems->setShortcut(tr("Ctrl+A"));
    connect(actionCheckAllItems, SIGNAL(triggered()), this, SLOT(checkAllItems()));

    actionUncheckItems = new QAction(tr("&Uncheck Selected Items"), this);
    //actionUncheckItems->setShortcut(tr("Alt+S"));
    connect(actionUncheckItems, SIGNAL(triggered()), this, SLOT(uncheckItems()));

    actionUncheckAllItems = new QAction(tr("&Uncheck All Items"), this);
    actionUncheckAllItems->setShortcut(tr("Ctrl+Shift+A"));
    connect(actionUncheckAllItems, SIGNAL(triggered()), this, SLOT(uncheckAllItems()));

    actionRemoveItems = new QAction(tr("&Remove Selected Items"), this);
    actionRemoveItems->setShortcut(Qt::Key_Delete);
    connect(actionRemoveItems, SIGNAL(triggered()), this, SLOT(removeItems()));

    actionRemoveAllItems = new QAction(tr("&Remove All Items"), this);
    //actionRemoveAllItems->setShortcut(tr("Ctrl+B"));
    connect(actionRemoveAllItems, SIGNAL(triggered()), this, SLOT(removeAllItems()));

    connect(pushCheck, SIGNAL(clicked()), this, SLOT(checkItems()));
    connect(pushCheckAll, SIGNAL(clicked()), this, SLOT(checkAllItems()));

    /* "Actions" actions */
    actionConvert = new QAction(tr("&Convert images"), this);
    actionConvert->setShortcut(tr("F5"));
    connect(actionConvert, SIGNAL(triggered()), this, SLOT(elabora()));

    actionInfo = new QAction(tr("&About..."), this);
    actionInfo->setShortcut(tr("F1"));
    connect(actionInfo, SIGNAL(triggered()), this, SLOT(about()));
}

void MainWindowImpl::setupMenu()
{
    menu_File->addAction(actionOpenFiles);
    menu_File->addAction(actionAddFiles);
    menu_File->addSeparator();
    menu_File->addAction(actionShowOptions);
    menu_File->addSeparator();
    menu_File->addAction(actionClose);

    menu_Edit->addAction(actionRemoveItems);
    menu_Edit->addAction(actionRemoveAllItems);
    menu_Edit->addSeparator();
    menu_Edit->addAction(actionCheckItems);
    menu_Edit->addAction(actionCheckAllItems);
    menu_Edit->addAction(actionUncheckItems);
    menu_Edit->addAction(actionUncheckAllItems);

    menu_Actions->addAction(actionConvert);

    menu_About->addAction(actionInfo);
}

void MainWindowImpl::createContextMenu()
{
    treeWidget->setContextMenuPolicy(Qt::ActionsContextMenu);
    treeWidget->addAction(actionCheckItems);
    treeWidget->addAction(actionCheckAllItems);
    treeWidget->addAction(actionUncheckItems);
    treeWidget->addAction(actionUncheckAllItems);
    treeWidget->addAction(actionRemoveItems);
    treeWidget->addAction(actionRemoveAllItems);
}

void MainWindowImpl::openFiles()
{
    iAList.clear();
    addFiles();
}

void MainWindowImpl::addFiles()
{
    QString path = QDir::homePath();
    if (settings->contains("General/Last Opened"))
        path = settings->value("General/Last Opened").toString();

    QStringList fileNames = QFileDialog::getOpenFileNames(this,
                                                          tr("Open Image"), path,
                                                          readableFiltersString);

    if (!fileNames.isEmpty())
        loadFiles(fileNames);
}

void MainWindowImpl::loadFiles(QStringList fileNames)
{
    treeWidget->clear();

    QFileInfo fi(fileNames.at(0));
    settings->setValue("General/Last Opened", fi.path());

    for (int i = 0; i < fileNames.count(); i++) {
        QFileInfo fi(fileNames.at(i));
        ImageAttributes iA;

        /* Controlla i doppioni! */
        if (!checkDuplicates(fileNames, i)) {
            iA.completeFileName = fileNames.at(i).toLocal8Bit();
            iA.fileName = fi.fileName();
            iA.suffix = fi.suffix();
            iA.size = fi.size();
            iA.path = fi.path();
            iAList << iA;   // aggiunge
        }
    }
    fillTreeView();
}

bool MainWindowImpl::checkDuplicates(QStringList fileNames, int e)
{
    int cnt = iAList.count();

    for (int i = 0; i < cnt; i++)
        if (iAList.at(i).completeFileName == fileNames.at(e).toLocal8Bit())
            return true;

    return false;
}

void MainWindowImpl::fillTreeView()
{
    int cnt = iAList.count();

    for (int i = 0; i < cnt; i++) {
        QTreeWidgetItem *item = new QTreeWidgetItem(treeWidget);
        item->setCheckState(0, Qt::Unchecked);
        item->setText(2, iAList.at(i).fileName);
        item->setText(3, iAList.at(i).suffix);
        item->setText(4, simplifyFileSize(iAList.at(i).size));
        item->setText(5, iAList.at(i).path);
    }
}

void MainWindowImpl::elabora()
{
    if (!iAList.isEmpty()) {
        if (areThereImagesChecked()) {
            if (!lineDirectory->text().isEmpty()) {
                QDir dir(lineDirectory->text());
                if((dir.exists()) || (checkSameDir->isChecked())) {
                    startConversion();
                }
                else {
                    int ret = QMessageBox::warning(0, tr("Warning"),
                                         tr("The output directory doesn't exists! Would you create it?"),
                                         QMessageBox::Ok | QMessageBox::Cancel);

                    if (ret == QMessageBox::Ok) {
                        if (!dir.mkpath(lineDirectory->text()))
                            QMessageBox::warning(0, tr("Warning"),
                                                 tr("The directory cannot be created!"));
                        else {
                            startConversion();
                        }
                    }
                }
            }
            else
                QMessageBox::warning(0, tr("Warning"),
                                         tr("Set an output directory!"));
        }
        else
            QMessageBox::warning(0, tr("Warning"),
                             tr("No image checked!"));
    }
    else
        QMessageBox::warning(0, tr("Warning"),
                             tr("No images to convert!"));
}

void MainWindowImpl::startConversion()
{
    settings->setValue("General/Save directory", lineDirectory->text());

    abort_all = false;

    statusBar()->showMessage(tr("Processing..."));
    curr_index = 0;

    convertThread->reset();

    if (!checkRename->isChecked()) {
        convertThread->setOverwrite(false);
        convertThread->enableRenamingString(false);
        convertThread->setOverwrite(overwriteMode());
    }
    else {
        convertThread->enableRenamingString(true);
        convertThread->setRenamingString(lineRename->text());   // Rinomina il file secondo #_copy
    }

    out_format = comboWFormats->currentText().split(" - ").at(0);
    out_path = lineDirectory->text();

    if (checkSameDir->isChecked())
        out_path = iAList.at(curr_index).path;

    quality = -1;

    if ((out_format == "JPG") || (out_format == "JPEG") || (out_format == "MPEG") || (out_format == "MPG")) {
        quality = jpgQuality;
    }

    if ((out_format == "PNG") || (out_format == "MNG")) {
        quality = pngQuality;
    }

    /* Change image size */
    if (groupDimensions->isChecked()) {
        QString aspectRatio = "!";

        if (checkRelative->isChecked())
            aspectRatio = "";

        // "%1 x %2 % !"

        QString resizingString;
        if (comboResizeValues->currentText() == "px")
            resizingString = QString("%1x%2%3")
            .arg(new_img_width)
            .arg(new_img_height)
            .arg(aspectRatio);
        if (comboResizeValues->currentText() == "%")
            resizingString = QString("%1x%2%")
            .arg(spin_geoWidth->value())
            .arg(spin_geoHeight->value());

        convertThread->setResize(resizingString);
    }

    /* FIXME */
    /* Change image density */
    if ((groupResolution->isChecked()) && ((m_xResolution != spin_resX->value()) || (m_yResolution != spin_resY->value()))) {
        QString densStr = QString("%1x%2")
                          .arg(spin_resX->value())
                          .arg(spin_resY->value());

        convertThread->setDensity(densStr);
    }
    convert();

    dlgCStatus->setup(iAList.count());
    dlgCStatus->exec();
}

void MainWindowImpl::convert()
{
    if ((treeWidget->topLevelItem(curr_index))->checkState(0)) {
        settings->setValue("General/Last Write Format", comboWFormats->currentIndex());
        statusBar()->showMessage(iAList.at(curr_index).fileName, 5000);
        convertThread->setGeneralParamethers(iAList.at(curr_index).completeFileName, out_format.toLower(), quality, out_path);
        convertThread->start();
    }
    else { // FIXME: caso in cui l'elemento non è checked
        nextConversion(2);
    }
}

void MainWindowImpl::nextConversion(int conv_status)
{
    dlgCStatus->step(conv_status, iAList.at(curr_index).fileName);

    if (!abort_all) {
        QString msg = "";
        if (conv_status == 1)
            msg = "Converted";
        else if (conv_status == -1)
            msg = "Error";

        (treeWidget->topLevelItem(curr_index))->setText(1, msg);

        curr_index++;
        if (curr_index < iAList.count()) {
            convert();
        }
        else
        {
            statusBar()->showMessage(tr("Processing finished!"), 5000);
        }
    }
}

void MainWindowImpl::loadFormats()
{
#ifdef Q_OS_LINUX

    list<CoderInfo> coderList;
    coderInfoList(&coderList,
                  CoderInfo::TrueMatch,
                  CoderInfo::AnyMatch,
                  CoderInfo::AnyMatch);

    list<CoderInfo>::iterator entry = coderList.begin();
    QStringList writableFormats;
    QString readableExts;

    while(entry != coderList.end())
    {
        // TODO: Mettere una lista con tutti i formati!!!

        if (entry->isReadable()) {
            readableFiltersString += QString(";;%3 [*.%1] (*.%1 *.%2 )")
            .arg(QString::fromStdString(entry->name()).toLower())   // Lower and upper filters for gnome compatibility (?)
            .arg(QString::fromStdString(entry->name()))
            .arg(QString::fromStdString(entry->description()));

            readableFomats << QString::fromStdString(entry->name()).toLower();

            readableExts += QString("*.%1 *.%2 ")
                            .arg(QString::fromStdString(entry->name()).toLower())
                            .arg(QString::fromStdString(entry->name()));
        }

        if (entry->isWritable())
            writableFormats << QString("%1 - (%2)")
            .arg(QString::fromStdString(entry->name()))
            .arg(QString::fromStdString(entry->description()));

        entry++;
    }
    readableFiltersString.prepend(tr("All Supported Formats (%1)").arg(readableExts));

#endif

#ifdef Q_OS_WIN32
	QFile data1("Windows\\ReadableFormats.txt");
	if (data1.open(QFile::ReadOnly)) {
		QTextStream in(&data1);

		readableFomats = in.readLine().split(" ");
		readableFiltersString = in.readLine();
	}

	QStringList writableFormats;
	QFile data2("Windows\\WritableFormats.txt");
	if (data2.open(QFile::ReadOnly)) {
		QTextStream in(&data2);
		QString line;
		do {
			line = in.readLine();
			writableFormats << line;
		} while (!line.isNull());
	}
	writableFormats.removeLast();
#endif

    comboWFormats->addItems(writableFormats);
	comboWFormats->setCurrentIndex(0);
}

void MainWindowImpl::setCurrentDirectory()
{
    bool enabled;

    if (checkSameDir->isChecked())
        enabled = false;
    else
        enabled = true;

    lineDirectory->setEnabled(enabled);
    bntSelDir->setEnabled(enabled);
}

void MainWindowImpl::openOutDirectory()
{
    QString dir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),
                                                    QDir::homePath(),
                                                    QFileDialog::ShowDirsOnly
                                                    | QFileDialog::DontResolveSymlinks);

    if (!dir.isNull())
        lineDirectory->setText(dir);
    // La cartella viene salvata quando viene avviata la procedura di conversione

}

void MainWindowImpl::loadOptions()
{
    QString iniPath = QDir::homePath() + "/" + INIFILENAME;

#ifdef Q_WS_WIN
    if ((QSysInfo::windowsVersion() == QSysInfo::WV_VISTA) || (QSysInfo::windowsVersion() == QSysInfo::WV_WINDOWS7)) {
        iniPath = QDir::homePath() + "/AppData/Roaming/Converseen/" + INIFILENAME;
    }
#endif

    settings = new QSettings(iniPath, QSettings::IniFormat);

    if (!settings->contains("General/Save directory"))
        lineDirectory->setText(QDir::homePath());
    else
        lineDirectory->setText(settings->value("General/Save directory").toString());

    if (settings->contains("Options/WindowSize"))
        restoreGeometry(settings->value("Options/WindowSize").toByteArray());

    comboWFormats->setCurrentIndex(settings->value("General/Last Write Format").toInt());

    checkOverwrite->setChecked(settings->value("Options/Overwrite mode").toBool());

    loadQuality();
}

void MainWindowImpl::setQuality()
{
    loadQuality();

    DialogQuality *dlg = new DialogQuality();
    dlg->setInitValues(jpgQuality, pngQuality);
    if (dlg->exec()) {
        jpgQuality = dlg->getJpegQuality();
        pngQuality = dlg->getPngQuality();

        settings->setValue("Quality/Jpeg quality", jpgQuality);
        settings->setValue("Quality/Png quality", pngQuality);
    }
}

void MainWindowImpl::loadQuality()
{
    if (settings->contains("Quality/Jpeg quality"))
        jpgQuality = settings->value("Quality/Jpeg quality").toInt();
    else
        jpgQuality = 85;

    if (settings->contains("Quality/Png quality"))
        pngQuality = settings->value("Quality/Png quality").toInt();
    else
        pngQuality = 7;
}

void MainWindowImpl::checkItems()
{
    QTreeWidgetItemIterator it(treeWidget);
    while (*it) {
        if ((*it)->isSelected())
            (*it)->setCheckState (0, Qt::Checked);
        ++it;
    }
}

void MainWindowImpl::checkAllItems()
{
    QTreeWidgetItemIterator it(treeWidget);
    while (*it) {
        (*it)->setCheckState (0, Qt::Checked);
        ++it;
    }
}

void MainWindowImpl::uncheckItems()
{
    QTreeWidgetItemIterator it(treeWidget);
    while (*it) {
        if ((*it)->isSelected())
            (*it)->setCheckState (0, Qt::Unchecked);
        ++it;
    }
}

void MainWindowImpl::uncheckAllItems()
{
    QTreeWidgetItemIterator it(treeWidget);
    while (*it) {
        (*it)->setCheckState (0, Qt::Unchecked);
        ++it;
    }
}

void MainWindowImpl::removeItems()
{
    for (int i = treeWidget->topLevelItemCount() - 1; i >= 0; i--)
        if ((treeWidget->topLevelItem(i))->isSelected()) {
             iAList.removeAt(i);
             treeWidget->takeTopLevelItem(i);
             resetDisplays();
         }
}

void MainWindowImpl::removeAllItems()
{
    treeWidget->clear();
    iAList.clear();

    resetDisplays();
}

void MainWindowImpl::relativeSizeW(double value)
{
    if (checkRelative->isChecked()) {
        if (comboResizeValues->currentText() == "px") {
            double relative_H = ( ((double)img_height * value) / (double)img_width);
            spin_geoHeight->setValue((double)qRound(relative_H));
        }
    }

    if (comboResizeValues->currentText() == "%") {
        if (checkRelative->isChecked())
            spin_geoHeight->setValue(spin_geoWidth->value());
    }

    showNewSizePreview();
}

void MainWindowImpl::relativeSizeH(double value)
{
    if (checkRelative->isChecked()) {
        if (comboResizeValues->currentText() == "px") {
            double relative_V = ( ((double)img_width * value) / (double)img_height);
            spin_geoWidth->setValue((double)qRound(relative_V));
        }
    }
    if (comboResizeValues->currentText() == "%") {
        if (checkRelative->isChecked())
            spin_geoWidth->setValue(spin_geoHeight->value());
    }
    showNewSizePreview();
}

QImage* MainWindowImpl::toQImage(const Image &image)
{
    QImage *newQImage = new QImage(image.columns(), image.rows(), QImage::Format_RGB32);

    const Magick::PixelPacket *pixels;
    Magick::ColorRGB rgb;
    for (int y = 0; y < newQImage->height(); y++) {
        pixels = image.getConstPixels(0, y, newQImage->width(), 1);
        for (int x = 0; x < newQImage->width(); x++) {
            rgb = (*(pixels + x));
            newQImage->setPixel(x, y, QColor((int) (255 * rgb.red())
                                             , (int) (255 * rgb.green())
                                             , (int) (255 * rgb.blue())).rgb());
        }
    }
    return newQImage;
}

void MainWindowImpl::loadPreview(QString fileName)
{
    Image my_image;
    try {
        my_image.read(fileName.toStdString());

        // Fills img_widht and img_height:
        img_width = my_image.columns();
        img_height = my_image.rows();

        // Load img resolution:
        m_xResolution = my_image.xResolution();
        m_yResolution = my_image.yResolution();

        if (checkShowPreview->isChecked()) {
            QImage *aQImage = toQImage(my_image);

            int n_heigth = (320 * img_height) / img_width;

            if ((img_width <= 320) && (img_height <= 240)) {
                    labelPreview->setPixmap(QPixmap::fromImage(*aQImage));
            }
            else
                if (img_width > img_height) {
                    labelPreview->setPixmap(QPixmap::fromImage(*aQImage).scaled(QSize(320, n_heigth),
                                                                                Qt::IgnoreAspectRatio,
                                                                                Qt::SmoothTransformation));

                }
                else if ((img_width <= img_height) || (n_heigth > 240)) {
                    int n_width = (240 * img_width) / img_height;
                    labelPreview->setPixmap(QPixmap::fromImage(*aQImage).scaled(QSize(n_width, 240),
                                                                                Qt::IgnoreAspectRatio,
                                                                                Qt::SmoothTransformation));
                }

            delete aQImage;
        }
        else
            labelPreview->setText(tr("No preview!"));
    }
    catch (Error& my_error) {
        labelPreview->setText(tr("Sorry! Selected image is damaged!"));
    }
    catch( Magick::WarningCoder &warning )
    {
        labelPreview->setText(tr("Sorry! Selected image is damaged!"));
    }
    catch( Magick::Warning &warning )
    {
        labelPreview->setText(tr("Sorry! Selected image is damaged!"));
    }
}

void MainWindowImpl::onItemSelection()
{
    if (!iAList.isEmpty()) {
        showImageInformations();
        showNewSizePreview();
    }
}

void MainWindowImpl::onPushResetClick()
{
    selectGeometryUnit(comboResizeValues->currentText());
    spin_resX->setValue(m_xResolution);
    spin_resY->setValue(m_yResolution);
}

void MainWindowImpl::showImageInformations()
{
    int index = treeWidget->currentIndex().row();
    loadPreview(iAList.at(index).completeFileName);

    labelType->setText(QString("<span style=\" font-size:8pt;\">%1</span>")
                       .arg(iAList.at(index).suffix));

    QString sSize = simplifyFileSize(iAList.at(index).size);
    labelFileSize->setText(QString("<span style=\" font-size:8pt;\">%1</span>")
                           .arg(sSize));

    label_ImageSize->setText(QString("<span style=\" font-size:8pt;\">%1 x %2 px</span>")
                             .arg(img_width)
                             .arg(img_height));

    label_ImageResolution->setText(QString("<span style=\" font-size:8pt;\">X = %1 Y = %2</span>")
                                   .arg(m_xResolution)
                                   .arg(m_yResolution));
}

void MainWindowImpl::editSettings()
{
    DialogOptions dlg;
    dlg.exec();

    loadOptions();
}

bool MainWindowImpl::overwriteMode()
{
    bool overwritingMode;

    if (settings->contains("Options/Overwrite mode"))
        overwritingMode = settings->value("Options/Overwrite mode").toBool();
    else
        overwritingMode = false;

    return overwritingMode;
}

QString MainWindowImpl::simplifyFileSize(qint64 size)
{
    QString sSize = "0";

    sSize = QString("%1 Bytes").arg(size);
    if (size > 1024)
        sSize = QString("%1 KiB").arg(QString::number( ((double)size / 1024), 'f', 1) );
    if (size / 1024 > 1024)
        sSize = QString("%1 MiB").arg(QString::number( ((double)size / 1024 / 1024), 'f', 1) );

    return sSize;
}

bool MainWindowImpl::areThereImagesChecked()
{
    QTreeWidgetItemIterator it(treeWidget);
    int i = 0;
    while (*it) {
        if ((*it)->checkState(0) == Qt::Checked)
            i++;
        ++it;
    }
    if (i > 0)
        return true;
    else
        return false;
}

void MainWindowImpl::about()
{
    DialogInfo dlg;
    dlg.exec();
}

void MainWindowImpl::enableRenameLine()
{
    if (checkRename->isChecked())
        lineRename->setEnabled(true);
    else
        lineRename->setEnabled(false);
}

void MainWindowImpl::enableShowPreview()
{
    if(treeWidget->selectedItems().count() > 0)
        onItemSelection();
    else
        labelPreview->setText(tr("No preview!"));
}

void MainWindowImpl::selectGeometryUnit(QString unit)
{
    if (unit == "px") {
        spin_geoWidth->setDecimals(0);
        spin_geoWidth->setSuffix(" px");
        spin_geoWidth->setValue((double)img_width);

        spin_geoHeight->setDecimals(0);
        spin_geoHeight->setSuffix(" px");
        spin_geoHeight->setValue((double)img_height);
    }
    if (unit == "%") {
        //spin_geoWidth->setDecimals(2);
        spin_geoWidth->setSuffix(" %");
        spin_geoWidth->setValue(100.00);

        //spin_geoHeight->setDecimals(2);
        spin_geoHeight->setSuffix(" %");
        spin_geoHeight->setValue(100.00);
    }
}

void MainWindowImpl::showNewSizePreview()
{
    if (comboResizeValues->currentText() == "%") {
        double new_width, new_height, round_new_width, round_new_height;

        new_width = (((double)img_width / 100) * (spin_geoWidth->value()));
        new_height = (((double)img_height / 100) * (spin_geoHeight->value()));

        round_new_width = qRound(new_width);
        round_new_height = qRound(new_height);


        labelPixelGeometry->setText(QString("<span style=\" font-size:8pt;\"><center>%1 x %2 pixels</center></span>")
                                    .arg(round_new_width)
                                    .arg(round_new_height));

        new_img_width = round_new_width;
        new_img_height = round_new_height;
    }
    if (comboResizeValues->currentText() == "px") {
        labelPixelGeometry->setText(QString("<span style=\" font-size:8pt;\"><center>%1 x %2 pixels</center></span>")
                                    .arg(spin_geoWidth->value())
                                    .arg(spin_geoHeight->value()));

        new_img_width = spin_geoWidth->value();
        new_img_height = spin_geoHeight->value();
    }
}

void MainWindowImpl::resetDisplays()
{
    labelPreview->setText(tr("No preview!"));

    img_width = 0;
    img_height = 0;

    new_img_width = 0;
    new_img_height = 0;

    m_xResolution = 0;
    m_yResolution = 0;

    spin_resX->setValue(m_xResolution);
    spin_resY->setValue(m_yResolution);

    if (comboResizeValues->currentText() == "%") {
        spin_geoWidth->setValue(100);
        spin_geoHeight->setValue(100);
    }
    if (comboResizeValues->currentText() == "px") {
        spin_geoWidth->setValue(1);
        spin_geoHeight->setValue(1);
    }

    labelType->setText(" - ");
    label_ImageSize->setText(" - ");
    labelFileSize->setText(" - ");
    label_ImageResolution->setText(" - ");

    labelPixelGeometry->setText("");
}

void MainWindowImpl::overwriteDialog(QString baseName)
{
    /*
        TODO: Sostituisci tutte (vedi sotto)
    */
    bool ok;
    QString newBaseName;

    while (newBaseName.isEmpty()) {
        newBaseName = QInputDialog::getText(0, tr("Suggest a new name!"),
                                             tr("New name:"), QLineEdit::Normal,
                                             baseName, &ok);

        if (!ok) {
            QMessageBox::StandardButton msg;
            msg = QMessageBox::question(this, tr("Please, specify a name!"),
            /* FIXME: tradurre meglio! */ tr("You should specify a name.\nClick Retry to specify a new name.\nClick Ignore to cancel this operation.\nClick Abort to cancel all operations."),
                            QMessageBox::Retry | QMessageBox::Ignore | QMessageBox::Abort);

            if (msg == QMessageBox::Ignore) {
                convertThread->stopProcess();
                break;
            }

            if (msg == QMessageBox::Abort) {
                stopProcess();
                break;
            }
        }
        else {
            if (newBaseName.isEmpty())
                QMessageBox::warning(0, tr("Warning!"),
                                 tr("The filename should not be empty!"));

            if (newBaseName == baseName) {
                QMessageBox::StandardButton msg;
                msg = QMessageBox::question(this, tr("Warning!"),
                                tr("The name chosen is identical to the initial one. Do you want to overwrite the original file?"),
                                QMessageBox::Yes | QMessageBox::No); // TODO: mettere si per tutte!

                if (msg == QMessageBox::No)
                    newBaseName.clear();
           }
       }
   }
   convertThread->mutex.unlock();
   convertThread->setNewBasename(newBaseName, ok);
}

void MainWindowImpl::saveOverwritingSettings()
{
    if (checkOverwrite->isChecked())
        settings->setValue("Options/Overwrite mode", true);
    else
        settings->setValue("Options/Overwrite mode", false);
}

void MainWindowImpl::stopProcess()
{
    convertThread->stopProcess();
    abort_all = true;
    statusBar()->showMessage(tr("Aborted!"), 5000);

    dlgCStatus->resetButtons();
}

void MainWindowImpl::errorMessage(QString err_status)   // Manda stringa di errore al dialog di conversione
{
    dlgCStatus->setErrorMsg(err_status);
}
